@@ -41,9 +41,7 @@
 #include "includes.h"
 RCSID("$OpenBSD: sshlogin.c,v 1.13 2004/08/12 09:18:24 djm Exp $");
 
-#include <util.h>
-#include <utmp.h>
-#include "sshlogin.h"
+#include "loginrec.h"
 #include "log.h"
 #include "buffer.h"
 #include "servconf.h"
@@ -60,38 +58,11 @@ u_long
 get_last_login_time(uid_t uid, const char *logname,
     char *buf, u_int bufsize)
 {
-	struct lastlog ll;
-	char *lastlog;
-	int fd;
-	off_t pos, r;
+	struct logininfo li;
 
-	lastlog = _PATH_LASTLOG;
-	buf[0] = '\0';
-
-	fd = open(lastlog, O_RDONLY);
-	if (fd < 0)
-		return 0;
-
-	pos = (long) uid * sizeof(ll);
-	r = lseek(fd, pos, SEEK_SET);
-	if (r == -1) {
-		error("%s: lseek: %s", __func__, strerror(errno));
-		return (0);
-	}
-	if (r != pos) {
-		debug("%s: truncated lastlog", __func__);
-		return (0);
-	}
-	if (read(fd, &ll, sizeof(ll)) != sizeof(ll)) {
-		close(fd);
-		return 0;
-	}
-	close(fd);
-	if (bufsize > sizeof(ll.ll_host) + 1)
-		bufsize = sizeof(ll.ll_host) + 1;
-	strncpy(buf, ll.ll_host, bufsize - 1);
-	buf[bufsize - 1] = '\0';
-	return ll.ll_time;
+	login_get_lastlog(&li, uid);
+	strlcpy(buf, li.hostname, bufsize);
+	return li.tv_sec;
 }
 
 /*
@@ -104,6 +75,7 @@ store_lastlog_message(const char *user, uid_t uid)
 	char *time_string, hostname[MAXHOSTNAMELEN] = "", buf[512];
 	time_t last_login_time;
 
+#ifndef NO_SSH_LASTLOG
 	if (!options.print_lastlog)
 		return;
 
@@ -122,6 +94,7 @@ store_lastlog_message(const char *user, uid_t uid)
 			    time_string, hostname);
 		buffer_append(&loginmsg, buf, strlen(buf));
 	}
+#endif /* NO_SSH_LASTLOG */
 }
 
 /*
@@ -132,51 +105,38 @@ void
 record_login(pid_t pid, const char *tty, const char *user, uid_t uid,
     const char *host, struct sockaddr * addr, socklen_t addrlen)
 {
-	int fd;
-	struct lastlog ll;
-	char *lastlog;
-	struct utmp u;
+	struct logininfo *li;
 
 	/* save previous login details before writing new */
 	store_lastlog_message(user, uid);
 
-	/* Construct an utmp/wtmp entry. */
-	memset(&u, 0, sizeof(u));
-	strncpy(u.ut_line, tty + 5, sizeof(u.ut_line));
-	u.ut_time = time(NULL);
-	strncpy(u.ut_name, user, sizeof(u.ut_name));
-	strncpy(u.ut_host, host, sizeof(u.ut_host));
-
-	login(&u);
-	lastlog = _PATH_LASTLOG;
-
-	/* Update lastlog unless actually recording a logout. */
-	if (strcmp(user, "") != 0) {
-		/*
-		 * It is safer to bzero the lastlog structure first because
-		 * some systems might have some extra fields in it (e.g. SGI)
-		 */
-		memset(&ll, 0, sizeof(ll));
-
-		/* Update lastlog. */
-		ll.ll_time = time(NULL);
-		strncpy(ll.ll_line, tty + 5, sizeof(ll.ll_line));
-		strncpy(ll.ll_host, host, sizeof(ll.ll_host));
-		fd = open(lastlog, O_RDWR);
-		if (fd >= 0) {
-			lseek(fd, (off_t) ((long) uid * sizeof(ll)), SEEK_SET);
-			if (write(fd, &ll, sizeof(ll)) != sizeof(ll))
-				logit("Could not write %.100s: %.100s", lastlog, strerror(errno));
-			close(fd);
-		}
-	}
+	li = login_alloc_entry(pid, user, host, tty);
+	login_set_addr(li, addr, addrlen);
+	login_login(li);
+	login_free_entry(li);
 }
 
+#ifdef LOGIN_NEEDS_UTMPX
+void
+record_utmp_only(pid_t pid, const char *ttyname, const char *user,
+		 const char *host, struct sockaddr * addr, socklen_t addrlen)
+{
+	struct logininfo *li;
+
+	li = login_alloc_entry(pid, user, host, ttyname);
+	login_set_addr(li, addr, addrlen);
+	login_utmp_only(li);
+	login_free_entry(li);
+}
+#endif
+
 /* Records that the user has logged out. */
 void
-record_logout(pid_t pid, const char *tty)
+record_logout(pid_t pid, const char *tty, const char *user)
 {
-	const char *line = tty + 5;	/* /dev/ttyq8 -> ttyq8 */
-	if (logout(line))
-		logwtmp(line, "", "");
+	struct logininfo *li;
+
+	li = login_alloc_entry(pid, user, NULL, tty);
+	login_logout(li);
+	login_free_entry(li);
 }
